@@ -1,14 +1,14 @@
 /*
  * smi.h --
  *
- *      Interface Definition of libsmi (version 2:25:0).
+ *      Interface Definition of libsmi (version 2:26:0).
  *
  * Copyright (c) 1999,2000 Frank Strauss, Technical University of Braunschweig.
  *
  * See the file "COPYING" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * @(#) $Id: smi.h.in 2013 2004-11-29 15:45:59Z strauss $
+ * @(#) $Id: smi.h.in 7734 2008-02-15 07:49:14Z schoenw $
  */
 
 #ifndef _SMI_H
@@ -16,6 +16,7 @@
 
 #include <stdlib.h>
 #include <stdarg.h>
+#include <stdint.h>
 #include <time.h>
 
 
@@ -24,13 +25,13 @@ extern "C" {
 #endif
 
 
-#define SMI_LIBRARY_VERSION "2:25:0"
+#define SMI_LIBRARY_VERSION "2:26:0"
 extern const char *smi_library_version;
 
 #define SMI_VERSION_MAJOR 0
 #define SMI_VERSION_MINOR 4
-#define SMI_VERSION_PATCHLEVEL 5
-#define SMI_VERSION_STRING "0.4.5"
+#define SMI_VERSION_PATCHLEVEL 7
+#define SMI_VERSION_STRING "0.4.7"
 extern const char *smi_version_string;
 
 
@@ -85,9 +86,19 @@ typedef enum SmiBasetype {
     SMI_BASETYPE_FLOAT64                = 8,  /* only SMIng                  */
     SMI_BASETYPE_FLOAT128               = 9,  /* only SMIng                  */
     SMI_BASETYPE_ENUM                   = 10,
-    SMI_BASETYPE_BITS                   = 11  /* SMIv2, SMIng and SPPI       */
+    SMI_BASETYPE_BITS                   = 11, /* SMIv2, SMIng and SPPI       */
+    SMI_BASETYPE_POINTER		= 12  /* only SMIng                  */
 } SmiBasetype;
 
+#define SMI_BASETYPE_INTEGER32_MIN  INT32_MIN
+#define SMI_BASETYPE_INTEGER32_MAX  INT32_MAX
+#define SMI_BASETYPE_INTEGER64_MIN  LIBSMI_INT64_MIN
+#define SMI_BASETYPE_INTEGER64_MAX  LIBSMI_INT64_MAX
+#define SMI_BASETYPE_UNSIGNED32_MIN 0
+#define SMI_BASETYPE_UNSIGNED32_MAX UINT32_MAX
+#define SMI_BASETYPE_UNSIGNED64_MIN 0
+#define SMI_BASETYPE_UNSIGNED64_MAX LIBSMI_UINT64_MAX
+
 /* SmiStatus -- values of status levels                                      */
 typedef enum SmiStatus {
     SMI_STATUS_UNKNOWN          = 0, /* should not occur                     */
@@ -108,7 +119,8 @@ typedef enum SmiAccess {
     SMI_ACCESS_READ_WRITE       = 5,
     SMI_ACCESS_INSTALL          = 6, /* these three entries are only valid   */
     SMI_ACCESS_INSTALL_NOTIFY   = 7, /* for SPPI                             */
-    SMI_ACCESS_REPORT_ONLY      = 8
+    SMI_ACCESS_REPORT_ONLY      = 8,
+    SMI_ACCESS_EVENT_ONLY      	= 9	 /* this entry is valid only for SMIng	 */
 } SmiAccess;
 
 /* SmiNodekind -- type or statement that leads to a definition               */
@@ -158,7 +170,11 @@ typedef enum SmiDecl {
     SMI_DECL_COLUMN             = 40,
     SMI_DECL_NOTIFICATION       = 41,
     SMI_DECL_GROUP              = 42,
-    SMI_DECL_COMPLIANCE         = 43
+    SMI_DECL_COMPLIANCE         = 43,
+    SMI_DECL_IDENTITY	        = 44,
+    SMI_DECL_CLASS		= 45,
+    SMI_DECL_ATTRIBUTE		= 46,
+    SMI_DECL_EVENT		= 47
 } SmiDecl;
 
 /* SmiIndexkind -- actual kind of a table row's index method                 */
@@ -231,9 +247,21 @@ typedef struct SmiMacro {
     SmiStatus           status;
     char                *description;
     char                *reference;
+    char		*abnf; //only for SMIng
 } SmiMacro;
 
+/* SmiIdentity -- the main structure of a SMIng Identity.		     */
+/* NOTE: Not to be confused with SMIv2 MODULE-IDENTITY */
+typedef struct SmiIdentity {
+    SmiIdentifier       name;
+    SmiDecl             decl;
+    SmiStatus           status;
+    char                *description;
+    char                *reference;
+} SmiIdentity;
+
 /* SmiType -- the main structure of a type definition (also base types)      */
+/* also SMIng attributes      */
 typedef struct SmiType {
     SmiIdentifier       name;
     SmiBasetype         basetype;
@@ -284,6 +312,39 @@ typedef struct SmiRefinement {
     char                *description;
 } SmiRefinement;
 
+/* SmiClass -- main structure for SMIng class statement               */
+typedef struct SmiClass {
+    SmiIdentifier       name;
+    SmiDecl             decl;
+    SmiStatus           status;
+    char                *description;
+    char                *reference;
+} SmiClass;
+
+/* SmiClass -- main structure for class attribute               */
+typedef struct SmiAttribute {
+    SmiIdentifier       name;
+    SmiBasetype         basetype;
+    SmiDecl             decl;
+    char                *format;
+    SmiValue            value;
+    char                *units;
+    SmiStatus           status;
+    char                *description;
+    char                *reference;
+    SmiAccess		access;
+} SmiAttribute;
+
+
+/* SmiEvent -- the main structure of a SMIng Event(part of class definition).*/
+typedef struct SmiEvent {
+    SmiIdentifier       name;
+    SmiDecl             decl;
+    SmiStatus           status;
+    char                *description;
+    char                *reference;
+} SmiEvent;
+
 
 
 extern int smiInit(const char *tag);
@@ -336,6 +397,21 @@ extern SmiRevision *smiGetNextRevision(SmiRevision *smiRevisionPtr);
 extern int smiGetRevisionLine(SmiRevision *smiRevisionPtr);
 
 
+
+extern SmiIdentity *smiGetFirstIdentity(SmiModule *smiModulePtr);
+
+extern SmiIdentity *smiGetNextIdentity(SmiIdentity *smiIdentityPtr);
+
+extern SmiIdentity *smiGetParentIdentity(SmiIdentity *smiIdentityPtr);
+
+extern int smiGetIdentityLine(SmiIdentity *smiIdentityPtr);
+
+extern SmiModule *smiGetIdentityModule(SmiIdentity *smiIdentityPtr);
+
+extern SmiIdentity *smiGetIdentity(SmiModule *smiModulePtr,char *identity);
+
+
+
 extern SmiType *smiGetType(SmiModule *smiModulePtr, char *type);
 
 extern SmiType *smiGetFirstType(SmiModule *smiModulePtr);
@@ -352,10 +428,62 @@ extern SmiRange *smiGetFirstRange(SmiType *smiTypePtr);
 
 extern SmiRange *smiGetNextRange(SmiRange *smiRangePtr);
 
+extern int smiGetMinMaxRange(SmiType *smiType, SmiValue *min, SmiValue *max);
+
 extern SmiNamedNumber *smiGetFirstNamedNumber(SmiType *smiTypePtr);
 
 extern SmiNamedNumber *smiGetNextNamedNumber(SmiNamedNumber
 					         *smiNamedNumberPtr);
+					         
+					         
+extern SmiClass *smiGetFirstClass(SmiModule *smiModulePtr);
+
+extern SmiClass *smiGetNextClass(SmiClass *smiClassPtr);
+
+extern SmiClass *smiGetParentClass(SmiClass *smiClassPtr);
+
+extern SmiModule *smiGetClassModule(SmiClass *smiClassPtr);
+
+extern SmiClass *smiGetClass(SmiModule *smiModulePtr,char *class);
+
+extern int smiGetClassLine(SmiClass *smiClassPtr);
+
+
+
+extern SmiAttribute *smiGetAttribute(SmiClass *smiClassPtr, char *attribute);
+
+extern SmiAttribute *smiGetFirstAttribute(SmiClass *smiClassPtr);
+
+extern SmiAttribute *smiGetNextAttribute(SmiAttribute *smiAtrributePtr);
+
+extern SmiType *smiGetAttributeParentType(SmiAttribute *smiAtrributePtr);
+
+extern SmiClass *smiGetAttributeParentClass(SmiAttribute *smiAtrributePtr);
+
+extern SmiAttribute *smiGetFirstUniqueAttribute(SmiClass *smiClassPtr);
+
+extern SmiAttribute *smiGetNextUniqueAttribute(SmiAttribute *smiTypePtr);
+
+extern int 		smiIsClassScalar(SmiClass *smiClassPtr);
+
+extern SmiNamedNumber *smiGetAttributeFirstNamedNumber(SmiAttribute *smiAttributePtr);
+
+extern SmiNamedNumber *smiGetAttributeNextNamedNumber(SmiNamedNumber
+					         *smiNamedNumberPtr);
+extern SmiRange *smiGetAttributeFirstRange(SmiAttribute *smiAttributePtr);
+
+extern SmiRange *smiGetAttributeNextRange(SmiRange *smiRangePtr);
+
+extern int smiGetAttributeLine(SmiAttribute *smiAttributePtr);
+
+
+extern SmiEvent *smiGetEvent(SmiClass *smiClassPtr, char *attribute);
+
+extern SmiEvent *smiGetFirstEvent(SmiClass *smiClassPtr);
+
+extern SmiEvent *smiGetNextEvent(SmiEvent *smiEventPtr);
+
+extern int smiGetEventLine(SmiEvent *smiEventPtr);
 
 
 extern SmiMacro *smiGetMacro(SmiModule *smiModulePtr, char *macro);
@@ -366,6 +494,7 @@ extern SmiMacro *smiGetNextMacro(SmiMacro *smiMacroPtr);
 
 extern SmiModule *smiGetMacroModule(SmiMacro *smiMacroPtr);
 
+extern int smiGetMacroLine(SmiMacro *smiMacroPtr);
 
 
 extern SmiNode *smiGetNode(SmiModule *smiModulePtr, const char *name);
@@ -450,6 +579,32 @@ extern char *smiRenderType(SmiType *smiTypePtr, int flags);
 #define SMI_UNKNOWN_LABEL "<unknown>"
 
    
+/*
+ * The functions smiGetMaxSize() and smiGetMinSize() compute the
+ * max size constraint on a given BITS, OCTET STRING or OBJECT
+ * IDENTIFIER type. The functions recurse towards the top of the
+ * type derivation tree.
+ */
+
+extern unsigned int smiGetMinSize(SmiType *smiType);
+extern unsigned int smiGetMaxSize(SmiType *smiType);
+
+/*
+ * Two utility functions to pack and unpack instance identifiers.
+ * The smiUnpack() function allocates the array of SmiValues and
+ * the smiPack() function allocates the array of SmiSubids.
+ */
+
+extern int smiUnpack(SmiNode *row, SmiSubid *oid, unsigned int oidlen,
+		     SmiValue **vals, int *valslen);
+
+extern int smiPack(SmiNode *row, SmiValue *vals, int valslen,
+		   SmiSubid **oid, unsigned int *oidlen);
+
+/*
+ * Two printf functions that allocate memory dynamically. The call has
+ * to free the allocated memory.
+ */
 
 extern int smiAsprintf(char **strp, const char *format, ...);
 
