@@ -10,7 +10,7 @@
  * See the file "COPYING" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * @(#) $Id: dump-tree.c 1571 2003-07-14 22:58:42Z schoenw $
+ * @(#) $Id: dump-tree.c 5755 2006-08-15 22:15:42Z schoenw $
  */
 
 #include <config.h>
@@ -473,6 +473,10 @@ static void dumpTree(int modc, SmiModule **modv, int flags, char *output)
 	    fprintf(f, "# united registration tree (generated by smidump "
 		    SMI_VERSION_STRING ")\n\n");
 	}
+	if (! (flags & SMIDUMP_FLAG_SILENT) && (flags & SMIDUMP_FLAG_ERROR)) {
+	    fprintf(f, "# WARNING: this output may be incorrect due to "
+		    "significant parse errors\n\n");
+	}
 	fprintTree(f);
 	
     } else {
@@ -486,6 +490,10 @@ static void dumpTree(int modc, SmiModule **modv, int flags, char *output)
 		fprintf(f, "# %s registration tree (generated by smidump "
 			SMI_VERSION_STRING ")\n\n", modv[i]->name);
 	    }
+	    if (! (flags & SMIDUMP_FLAG_SILENT) && (flags & SMIDUMP_FLAG_ERROR)) {
+		fprintf(f, "# WARNING: this output may be incorrect due to "
+			"significant parse errors\n\n");
+	    }
 	    fprintTree(f);
 	}
     }
