@@ -9,7 +9,7 @@
  * See the file "COPYING" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * @(#) $Id: smidump.c 3012 2005-11-14 16:23:17Z schoenw $
+ * @(#) $Id: smidump.c 7285 2007-09-27 19:48:50Z schoenw $
  */
 
 #include <config.h>
@@ -44,6 +44,7 @@ static void format(char *form);
 static int flags;
 static int mFlag = 0;	/* show the name for error messages */
 static int sFlag = 0;	/* show the severity for error messages */
+static int kFlag = 0;	/* keep going after severe errors */
 static SmidumpDriver *driver;
 static SmidumpDriver *firstDriver;
 static SmidumpDriver *lastDriver;
@@ -66,6 +67,7 @@ static optStruct genericOpt[] = {
     { 'u', "unified",        OPT_FLAG,   unified,       OPT_CALLFUNC },
     { 'f', "format",         OPT_STRING, format,        OPT_CALLFUNC },
     { 'o', "output",         OPT_STRING, &output,       0            },
+    { 'k', "keep-going",     OPT_FLAG,	 &kFlag,	0 },
     { 0, 0, OPT_END, 0, 0 }  /* no more options */
 };
 
@@ -78,6 +80,7 @@ void *xmalloc(size_t size)
 	fprintf(stderr, "smidump: malloc failed - running out of memory\n");
 	exit(1);
     }
+
     return m;
 }
 
@@ -95,6 +98,18 @@ void *xrealloc(void *ptr, size_t size)
 
 
 
+void *xcalloc(size_t nmemb, size_t size)
+{
+    char *m = calloc(nmemb, size);
+    if (! m) {
+	fprintf(stderr, "smidump: calloc failed - running out of memory\n");
+	exit(1);
+    }
+    return m;
+}
+
+
+
 char *xstrdup(const char *s)
 {
     char *m = strdup(s);
@@ -173,7 +188,8 @@ static void usage()
 	    "  -s, --severity       print the severity of errors in brackets\n"
 	    "  -f, --format=format  use <format> when dumping (default %s)\n"
 	    "  -o, --output=name    use <name> when creating names for output files\n"
-	    "  -u, --unified        print a single unified output of all modules\n\n",
+	    "  -u, --unified        print a single unified output of all modules\n"
+	    "  -k, --keep-going     continue after serious parse errors\n\n",
 	    defaultDriver ? defaultDriver->name : "none");
 
     fprintf(stderr, "Supported formats are:\n");
@@ -299,9 +315,6 @@ int main(int argc, char *argv[])
     initPython();
     initSming();
     initSmi();                defaultDriver = lastDriver;
-#if 0
-    initSmiv3();
-#endif
     initSppi();
 #if 0
     initSql();
@@ -314,6 +327,7 @@ int main(int argc, char *argv[])
     initXml();
     initXsd();
     initCompliances();
+    initYang();
     
     for (i = 1; i < argc; i++)
 	if ((strstr(argv[i], "-c") == argv[i]) ||
@@ -363,6 +377,7 @@ int main(int argc, char *argv[])
 	smiModule = modulename ? smiGetModule(modulename) : NULL;
 	if (smiModule) {
 	    if ((smiModule->conformance) && (smiModule->conformance < 3)) {
+		flags |= SMIDUMP_FLAG_ERROR;
 		if (! (flags & SMIDUMP_FLAG_SILENT)) {
 		    fprintf(stderr,
 			    "smidump: module `%s' contains errors, "
@@ -377,7 +392,15 @@ int main(int argc, char *argv[])
 	}
     }
 
-    (driver->func)(modc, modv, flags, output);
+    if (! (flags & SMIDUMP_FLAG_ERROR) || kFlag) {
+	(driver->func)(modc, modv, flags, output);
+    } else {
+	if (! (flags & SMIDUMP_FLAG_SILENT)) {
+	    fprintf(stderr,
+		    "smidump: aborting due to severe parsing errors\n"
+		    "smidump: use the -k option to force continuation\n");
+	}
+    }
 
     smiExit();
 
